@@ -119,5 +119,5 @@ instance AWSRequest ListApplications where
 
 instance FromJSON ListApplicationsResponse where
     parseJSON = withObject "ListApplicationsResponse" $ \o -> ListApplicationsResponse
-        <$> o .:  "applications"
+        <$> o .:? "applications" .!= mempty
         <*> o .:? "nextToken"
