@@ -7,6 +7,7 @@
 {-# LANGUAGE OverloadedStrings           #-}
 {-# LANGUAGE RecordWildCards             #-}
 {-# LANGUAGE TypeFamilies                #-}
+{-# LANGUAGE ViewPatterns                #-}
 
 {-# OPTIONS_GHC -fno-warn-unused-imports #-}
 
@@ -215,8 +216,6 @@ module Network.AWS.CodeDeploy.Types
     , EC2TagFilterType (..)
     ) where
 
-import Data.Char (isUpper)
-import Network.AWS.Error
 import Network.AWS.Prelude
 import Network.AWS.Signing
 import qualified GHC.Exts
@@ -228,15 +227,39 @@ instance AWSService CodeDeploy where
     type Sg CodeDeploy = V4
     type Er CodeDeploy = JSONError
 
-    service = Service
-        { _svcAbbrev       = "CodeDeploy"
-        , _svcPrefix       = "codedeploy"
-        , _svcVersion      = "2014-10-06"
-        , _svcTargetPrefix = Just "CodeDeploy_20141006"
-        , _svcJSONVersion  = Just "1.1"
-        }
+    service = service'
+      where
+        service' :: Service CodeDeploy
+        service' = Service
+            { _svcAbbrev       = "CodeDeploy"
+            , _svcPrefix       = "codedeploy"
+            , _svcVersion      = "2014-10-06"
+            , _svcTargetPrefix = Just "CodeDeploy_20141006"
+            , _svcJSONVersion  = Just "1.1"
+            , _svcHandle       = handle
+            , _svcRetry        = retry
+            }
 
-    handle = jsonError statusSuccess
+        handle :: Status
+               -> Maybe (LazyByteString -> ServiceError JSONError)
+        handle = jsonError statusSuccess service'
+
+        retry :: Retry CodeDeploy
+        retry = Exponential
+            { _retryBase     = 0.05
+            , _retryGrowth   = 2
+            , _retryAttempts = 5
+            , _retryCheck    = check
+            }
+
+        check :: Status
+              -> JSONError
+              -> Bool
+        check (statusCode -> s) (awsErrorCode -> e)
+            | s == 500  = True -- General Server Error
+            | s == 509  = True -- Limit Exceeded
+            | s == 503  = True -- Service Unavailable
+            | otherwise = False
 
 data GenericRevisionInfo = GenericRevisionInfo
     { _griDeploymentGroups :: List "deploymentGroups" Text
@@ -293,7 +316,7 @@ griRegisterTime = lens _griRegisterTime (\s a -> s { _griRegisterTime = a }) . m
 
 instance FromJSON GenericRevisionInfo where
     parseJSON = withObject "GenericRevisionInfo" $ \o -> GenericRevisionInfo
-        <$> o .:  "deploymentGroups"
+        <$> o .:? "deploymentGroups" .!= mempty
         <*> o .:? "description"
         <*> o .:? "firstUsedTime"
         <*> o .:? "lastUsedTime"
@@ -535,7 +558,7 @@ instance FromJSON InstanceSummary where
         <$> o .:? "deploymentId"
         <*> o .:? "instanceId"
         <*> o .:? "lastUpdatedAt"
-        <*> o .:  "lifecycleEvents"
+        <*> o .:? "lifecycleEvents" .!= mempty
         <*> o .:? "status"
 
 instance ToJSON InstanceSummary where
@@ -672,11 +695,11 @@ dgiTargetRevision =
 instance FromJSON DeploymentGroupInfo where
     parseJSON = withObject "DeploymentGroupInfo" $ \o -> DeploymentGroupInfo
         <$> o .:? "applicationName"
-        <*> o .:  "autoScalingGroups"
+        <*> o .:? "autoScalingGroups" .!= mempty
         <*> o .:? "deploymentConfigName"
         <*> o .:? "deploymentGroupId"
         <*> o .:? "deploymentGroupName"
-        <*> o .:  "ec2TagFilters"
+        <*> o .:? "ec2TagFilters" .!= mempty
         <*> o .:? "serviceRoleArn"
         <*> o .:? "targetRevision"
 
